/**********************************************************************/
/*  PROGRAM: MQCL.C                                                   */
/*  PURPOSE: MQSeries Windows Client sample C source                  */
/*  AUTHOR : Paul Clarke                                              */
/**********************************************************************/
                                       /* Include files               */
#include "windows.h"
#include "mqcl.h"
#include <stdlib.h>
#include <memory.h>
#include <string.h>
#include <cmqc.h>
#include <process.h>
                                       /* Constants                   */
#define MQCONN_TABLE_SIZE   10
#define HELP_FILE_NAME_SIZE 140
#define WM_USER_HELP        (WM_USER+100)
/**********************************************************************/
/* Global statics                                                     */
/**********************************************************************/
HWND    hMain;
int     Mains = 0;
MQHCONN MQCONNTable[MQCONN_TABLE_SIZE];
char    HelpFileName[HELP_FILE_NAME_SIZE];
FARPROC NextMessageFilterProc;
FARPROC MessageFilter;
WORD    CurrentMenuCmd;
DWORD   CurrentMenuBits;
LPSTR   GlobalApi;
MQLONG  GlobalCompCode;
MQLONG  GlobalReason;

/**********************************************************************/
/* Local Datatypes                                                    */
/**********************************************************************/
typedef struct _Words
{
  HWND     hWnd;
  HANDLE   hInstance;
  MQHCONN  hConn;
  char     QMName[50];
  char     ActualQMName[50];
  char     GetQName[50];
  char     PutQName[50];
  int      PutSync;
  int      GetSync;
  short    xChar;
  short    yChar;
} WORDS;

/**********************************************************************/
/* Local functions                                                    */
/**********************************************************************/
int      PASCAL WinMain    (HANDLE  hInstance,
                            HANDLE  hPrevInstance,
                            LPSTR   lpCmdLine,
                            int     nCmdShow);

long FAR PASCAL MainWndProc(HWND    hWnd,
                            UINT    message,
                            UINT    wParam,
                            LONG    lParam);

BOOL FAR PASCAL MQCONNDlg  (HWND    hDlg,
                            UINT    message,
                            UINT    wParam,
                            LONG    lParam);

BOOL FAR PASCAL MQPUTDlg   (HWND    hDlg,
                            UINT    message,
                            UINT    wParam,
                            LONG    lParam);

BOOL FAR PASCAL MQGETDlg   (HWND    hDlg,
                            UINT    message,
                            UINT    wParam,
                            LONG    lParam);

BOOL FAR PASCAL AboutDlg   (HWND    hDlg,
                            UINT    message,
                            UINT    wParam,
                            LONG    lParam);

BOOL FAR PASCAL MQIErrorDlg(HWND hDlg,
                            UINT message,
                            UINT wParam,
                            LONG lParam);

DWORD FAR PASCAL HelpMessageFilterHook(int   nCode,
                                       WORD  wParam,
                                       LPMSG lpMsg);

static void     MQIError   (WORDS * pWords,
                            HWND    hWnd,
                            LPSTR   Api,
                            MQLONG  CompCode,
                            MQLONG  Reason);

static void     SetTitle   (LPSTR   Title,
                            HWND    hwnd,
                            WORDS * pWords);

static void     SetMenus   (WORDS * pWords);

static void     GetHelpName(HANDLE  hInstance,
                            char  * HelpFileName);
/**********************************************************************/
/* FUNCTION: WinMain                                                  */
/* PURPOSE:  Main program, windows message loop                       */
/**********************************************************************/
int PASCAL WinMain(HANDLE hInstance,
                   HANDLE hPrevInstance,
                   LPSTR  lpCmdLine,
                   int    nCmdShow)
{
  WNDCLASS  wc;
  MSG       msg;
  WORDS   * pWords;
  short     xScreen,yScreen;
  MQLONG    CompCode,Reason;
  int       i;

  if (!hPrevInstance)
  {
    wc.style         = CS_GLOBALCLASS;
    wc.lpfnWndProc   = MainWndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = sizeof(WORDS *);
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(hInstance, MAKEINTRESOURCE(2));
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName  = "MQCLMenu";
    wc.lpszClassName = "MQCLClass";

    if (!RegisterClass(&wc)) return FALSE;
  }
                                       /* How big is the screen       */
  xScreen = GetSystemMetrics (SM_CXSCREEN);
  yScreen = GetSystemMetrics (SM_CYSCREEN);
                                       /* Create a window             */
  hMain = CreateWindow("MQCLClass",
                       "MQSeries Client",
                       WS_OVERLAPPEDWINDOW,
                       xScreen / 3,
                       yScreen / 3,
                       xScreen / 3,
                       yScreen / 3,
                       NULL,
                       NULL,
                       hInstance,
                       NULL);
  if (!hMain) return (FALSE);

  pWords = (WORDS *)GetWindowWord(hMain, 0);
  pWords -> hInstance = hInstance;

  GetHelpName(hInstance, HelpFileName);
  ShowWindow(hMain, nCmdShow);
  UpdateWindow(hMain);
                                       /* Initialise MQCONN Table     */
  for (i=0; i<MQCONN_TABLE_SIZE; i++) MQCONNTable[i]=MQHC_UNUSABLE_HCONN;

                                       /* Install help hook            */
  MessageFilter=MakeProcInstance((FARPROC)HelpMessageFilterHook, hInstance);
  NextMessageFilterProc=SetWindowsHook(WH_MSGFILTER, MessageFilter);
                                      /* Do message loop               */
  while (GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }
                                       /* Free Help Hook               */
  UnhookWindowsHook(WH_MSGFILTER, MessageFilter);
  FreeProcInstance(MessageFilter);
                                       /* Disconnect any HCONNs       */
  for (i=0; i<MQCONN_TABLE_SIZE; i++)
  {
    if (MQCONNTable[i] != MQHC_UNUSABLE_HCONN)
      MQDISC(&MQCONNTable[i], &CompCode, &Reason);
  }
                                       /* Return any error to Windows */
  return (msg.wParam);
}

/**********************************************************************/
/*  FUNCTION: MainWndProc                                             */
/*  PURPOSE:  Process the main window messages                        */
/**********************************************************************/
long FAR PASCAL MainWndProc(HWND hWnd,
                            UINT message,
                            UINT wParam,
                            LONG lParam)
{
  HWND        hNewMain;
  MQLONG      CompCode, Reason;
  WORDS     * pWords, * pNewWords;
  FARPROC     lpProc;
  RECT        rect;
  HDC         hDC;
  PAINTSTRUCT ps;
  TEXTMETRIC  tm;
  int         line;
  char        Buffer[100];
  MQHCONN     hConn;
  int         i;

  switch (message)
  {
    case WM_CREATE:
                                       /* Get some window words       */
         pWords = (WORDS *) malloc(sizeof(WORDS));
         memset(pWords, 0, sizeof(WORDS));
         pWords -> hConn = MQHC_UNUSABLE_HCONN;
         pWords -> hWnd  = hWnd;
         SetWindowWord( hWnd, 0, (WORD) pWords );
                                       /* Init windowy bits           */
         hDC = GetDC(hWnd);
         GetTextMetrics(hDC, &tm);
         pWords -> xChar = tm.tmAveCharWidth;
         pWords -> yChar = tm.tmHeight + tm.tmExternalLeading;
         ReleaseDC(hWnd, hDC);
         Mains++;
                                       /* Set the inital menu state   */
         SetMenus(pWords);
         break;

    case WM_USER_HELP:
         if (wParam==MSGF_MENU                         &&
             !(LOWORD(CurrentMenuBits) & MF_POPUP)   &&
             !(LOWORD(CurrentMenuBits) & MF_SYSMENU))
         {
           WinHelp(hWnd,HelpFileName,HELP_CONTEXT, CurrentMenuCmd);
         }
         break;

    case WM_MENUSELECT:
         CurrentMenuBits = lParam;
         CurrentMenuCmd  = wParam;
         return (DefWindowProc(hWnd, message, wParam, lParam));
         break;

    case WM_KEYDOWN:
                                       /* Check for help request     */
         if (wParam == VK_F1)
           WinHelp(hWnd,HelpFileName,HELP_INDEX,0L);
         else
           return (DefWindowProc(hWnd, message, wParam, lParam));
         break;

    case WM_COMMAND:
         pWords = (WORDS *) GetWindowWord( hWnd, 0 );
         switch (wParam)
         {
           case IDM_EXIT:
                PostQuitMessage(0);
                break;

           case IDM_NEW:
                GetWindowRect( hWnd, &rect );

                hNewMain = CreateWindow("MQCLClass",
                                        "MQSeries Client",
                                        WS_OVERLAPPEDWINDOW,
                                        rect.left + 10,
                                        rect.top  + 10,
                                        rect.right   - rect.left,
                                        rect.bottom  - rect.top,
                                        NULL,
                                        NULL,
                                        pWords -> hInstance,
                                        NULL);
                if (hNewMain)
                {
                                       /* Get the window words        */
                  pNewWords = (WORDS *) GetWindowWord(hNewMain, 0);
                  pNewWords -> hInstance = pWords -> hInstance;

                  ShowWindow   ( hNewMain, SW_SHOW);
                  UpdateWindow ( hNewMain);
                }
                break;

           case IDM_MQCONN:
                lpProc = MakeProcInstance(MQCONNDlg, pWords -> hInstance);

                DialogBox(pWords -> hInstance,
                          "MQCONN",
                          hWnd,
                          lpProc);

                FreeProcInstance(lpProc);
                                       /* Update the menus            */
                SetMenus(pWords);
                break;

           case IDM_MQDISC:
                hConn = pWords -> hConn;
                MQDISC(&pWords -> hConn, &CompCode, &Reason);
                if (CompCode)
                  MQIError(pWords,hWnd,"MQDISC", CompCode, Reason);
                else
                {
                                       /* Remove from MQCONN Table    */
                  for (i=0; i<MQCONN_TABLE_SIZE; i++)
                  {
                    if (MQCONNTable[i] == hConn)
                    {
                      MQCONNTable[i] = MQHC_UNUSABLE_HCONN;
                      break;
                    }
                  }
                }
                                       /* Update the menus            */
                SetMenus(pWords);
                break;

           case IDM_MQPUT:
                lpProc = MakeProcInstance(MQPUTDlg, pWords -> hInstance);

                DialogBox(pWords -> hInstance,
                          "MQPUT",
                          hWnd,
                          lpProc);

                FreeProcInstance(lpProc);
                break;

           case IDM_MQGET:
                lpProc = MakeProcInstance(MQGETDlg, pWords -> hInstance);

                DialogBox(pWords -> hInstance,
                          "MQGET",
                          hWnd,
                          lpProc);

                FreeProcInstance(lpProc);
                break;

           case IDM_MQCMIT:
                MQCMIT(pWords -> hConn,
                       &CompCode,
                       &Reason);
                if (CompCode) MQIError(pWords,hWnd,"MQCMIT", CompCode, Reason);
                break;


           case IDM_MQBACK:
                MQBACK(pWords -> hConn,
                       &CompCode,
                       &Reason);
                if (CompCode) MQIError(pWords,hWnd,"MQBACK", CompCode, Reason);
                break;

          case IDM_HELP_INDEX:
               WinHelp(hWnd,HelpFileName,HELP_INDEX,0L);
               break;

          case IDM_HELP_HELP:
               WinHelp(hWnd,"WINHELP.HLP",HELP_INDEX,0L);
               break;

          case IDM_ABOUT:
               lpProc = MakeProcInstance(AboutDlg, pWords -> hInstance);

               DialogBox(pWords -> hInstance,
                          "About",
                          hWnd,
                          lpProc);

              FreeProcInstance(lpProc);
              break;

           default:
                return (DefWindowProc(hWnd, message, wParam, lParam));
         }
         InvalidateRect( hWnd, NULL, TRUE );
         break;

    case WM_PAINT:
         pWords = (WORDS *) GetWindowWord( hWnd, 0 );
         hDC = BeginPaint( hWnd, &ps );
         line = 0;
                                       /* Write out window contents   */
         wsprintf(Buffer,"Instance: %X",pWords -> hInstance);
         TextOut(hDC,pWords->xChar,++line*pWords->yChar,Buffer,strlen(Buffer));

         wsprintf(Buffer,"HCONN: %ld",pWords -> hConn);
         TextOut(hDC,pWords->xChar,++line*pWords->yChar,Buffer,strlen(Buffer));

         wsprintf(Buffer,"QM: '%s'",(LPSTR)pWords -> QMName);
         TextOut(hDC,pWords->xChar,++line*pWords->yChar,Buffer,strlen(Buffer));

         wsprintf(Buffer,"Actual QM: '%s'",(LPSTR)pWords -> ActualQMName);
         TextOut(hDC,pWords->xChar,++line*pWords->yChar,Buffer,strlen(Buffer));

         EndPaint(hWnd, &ps);
         break;

    case WM_DESTROY:
         Mains--;
         if (Mains == 0) PostQuitMessage(0);
                                       /* DELIBERATELY NO BREAK HERE  */
    default:
         return (DefWindowProc(hWnd, message, wParam, lParam));

  }

  return (NULL);
}

/**********************************************************************/
/*  FUNCTION: MQCONNDlg                                               */
/*  PURPOSE:  Connect dialog                                          */
/**********************************************************************/
BOOL FAR PASCAL MQCONNDlg(HWND hDlg,
                          UINT message,
                          UINT wParam,
                          LONG lParam)
{
  HWND    hOwner;
  WORDS * pWords;
  MQOD    ObjDesc    = { MQOD_DEFAULT };
  MQHOBJ  hObj;
  MQLONG  CompCode, Reason;
  MQLONG  Options    = MQOO_INQUIRE;
  MQLONG  Selectors[1];
  int     i;

  switch (message)
  {
    case WM_INITDIALOG:
                                       /* Set max size of field       */
         SendDlgItemMessage(hDlg,
                            IDD_QM,
                            EM_LIMITTEXT,
                      (int) MQ_Q_MGR_NAME_LENGTH,
                            0L);
                                       /* Find the Window words       */
         hOwner = GetWindow( hDlg, GW_OWNER );
         pWords = (WORDS *) GetWindowWord( hOwner, 0 );
                                       /* Set the field in the Dlg    */
         SetDlgItemText(hDlg,
                        IDD_QM,
                        pWords -> QMName);
         return (TRUE);
         break;

    case WM_USER_HELP:
         WinHelp(hDlg,HelpFileName,HELP_CONTEXT,IDM_MQCONN);
         break;


    case WM_COMMAND:
         switch (wParam)
         {
           case IDOK:
                                       /* Find the Window words       */
                hOwner = GetWindow( hDlg, GW_OWNER );
                pWords = (WORDS *) GetWindowWord( hOwner, 0 );
                                       /* Get the field from the Dlg  */
                GetDlgItemText(hDlg,
                               IDD_QM,
                               pWords -> QMName,
                               sizeof(pWords -> QMName));
                                       /* Issue the MQCONN            */
                SetTitle("Connecting...",hDlg,pWords);

                                       /* Disable buttons             */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL),FALSE);
                EnableWindow(GetDlgItem(hDlg,IDOK)    ,FALSE);

                MQCONN (pWords -> QMName,
                       &pWords -> hConn,
                       &CompCode,
                       &Reason);
                                       /* Enable buttons              */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL),TRUE);
                EnableWindow(GetDlgItem(hDlg,IDOK)    ,TRUE);

                switch(CompCode)
                {
                  case MQCC_FAILED:
                       MQIError(pWords,hDlg,"MQCONN", CompCode, Reason);
                       break;

                  case MQCC_OK:
                                       /* Add to MQCONN table         */
                       for (i=0; i<MQCONN_TABLE_SIZE; i++)
                       {
                         if (MQCONNTable[i] == MQHC_UNUSABLE_HCONN)
                         {
                           MQCONNTable[i] = pWords -> hConn;
                           break;
                         }
                       }
                                       /* DELIBERATELY NO BREAK HERE  */
                  default:
                       ObjDesc.ObjectType = MQOT_Q_MGR;
                                            /* Get the real QM name        */
                       MQOPEN (pWords -> hConn,
                               &ObjDesc,
                               Options,
                               &hObj,
                               &CompCode,
                               &Reason);

                       if (CompCode)
                       {
                         MQIError(pWords,hDlg,"MQOPEN", CompCode, Reason);
                         break;
                       }

                       Selectors[0] = MQCA_Q_MGR_NAME;

                       MQINQ (pWords -> hConn,
                              hObj,
                              1,
                              Selectors,
                              0,
                              NULL,
                              MQ_Q_MGR_NAME_LENGTH,
                              pWords -> ActualQMName,
                              &CompCode,
                              &Reason);
                       if (CompCode) MQIError(pWords,hDlg,"MQINQ", CompCode, Reason);

                       MQCLOSE(pWords -> hConn,
                              &hObj,
                               0L,
                              &CompCode,
                              &Reason);
                       if (CompCode) MQIError(pWords,hDlg,"MQCLOSE", CompCode, Reason);

                       EndDialog(hDlg, TRUE);
                       break;
                }
                return (TRUE);

           case IDCANCEL:
                EndDialog(hDlg, FALSE);
                return (TRUE);
         }
         break;

    default:
         break;
  }
  return (FALSE);
}

/**********************************************************************/
/*  FUNCTION: MQPutDlg                                                */
/*  PURPOSE:  Put dialog                                              */
/**********************************************************************/
BOOL FAR PASCAL MQPUTDlg(HWND hDlg,
                         UINT message,
                         UINT wParam,
                         LONG lParam)
{
  HWND    hOwner;
  WORDS * pWords;
  MQLONG  CompCode, Reason;
  MQOD    ObjDesc    = { MQOD_DEFAULT };
  MQMD    MsgDesc    = { MQMD_DEFAULT };
  MQPMO   PutMsgOpts = { MQPMO_DEFAULT };
  MQLONG  Options    = MQOO_OUTPUT;
  MQHOBJ  hObj;
  char    CharReps[10] = "1";
  int     reps;
  char    Msg[100];
  int     Count;
  char    Buffer[50];
  char  * pMsg       = NULL;
  int     AllocatedStorage;
  long    CheckSum;
  MQLONG  MsgLen;
  long    i;

  switch (message)
  {
    case WM_INITDIALOG:
                                       /* Find the Window words       */
         hOwner = GetWindow( hDlg, GW_OWNER );
         pWords = (WORDS *) GetWindowWord( hOwner, 0 );
         SetTitle("Put",hDlg,pWords);
                                       /* Set max size of field       */
         SendDlgItemMessage(hDlg,
                            IDD_Q,
                            EM_LIMITTEXT,
                      (int) MQ_Q_NAME_LENGTH,
                            0L);
                                       /* Set the field in the Dlg    */
         SetDlgItemText(hDlg,
                        IDD_Q,
                        pWords -> PutQName);
         SetDlgItemText(hDlg,
                        IDD_REPS,
                        CharReps);

         SendDlgItemMessage(hDlg,
                            IDD_SYNCPOINT,
                            BM_SETCHECK, pWords -> PutSync,0);

         return (TRUE);
         break;

    case WM_USER_HELP:
         WinHelp(hDlg,HelpFileName,HELP_CONTEXT,IDM_MQPUT);
         break;

    case WM_COMMAND:
         switch (wParam)
         {
           case IDOK:
                Count = 0;
                                       /* Find the Window words       */
                hOwner = GetWindow( hDlg, GW_OWNER );
                pWords = (WORDS *) GetWindowWord( hOwner, 0 );
                SetTitle("Putting...",hDlg,pWords);
                                       /* Get the fields from the Dlg */
                GetDlgItemText(hDlg,
                               IDD_Q,
                               pWords -> PutQName,
                               sizeof(pWords -> PutQName));

                GetDlgItemText(hDlg,
                               IDD_MSG,
                               Msg,
                               sizeof(Msg));

                GetDlgItemText(hDlg,
                               IDD_REPS,
                               CharReps,
                               sizeof(CharReps));

                pWords->PutSync = (int)SendDlgItemMessage(hDlg,
                                                          IDD_SYNCPOINT,
                                                          BM_GETCHECK, 0,0);
                /******************************************************/
                /* If the user has requested a large message.         */
                /* In this case the message with have the format :-   */
                /*   Identifier string            "[CK]"              */
                /*   Checksum of following chars  <long>              */
                /*   Random characters            <MsgLen bytes>      */
                /******************************************************/
                if (Msg[0] == '#')
                {
                  MsgLen = atol(&Msg[1]) + 8;
                  if (MsgLen < 50) MsgLen = 50;
                  pMsg   = malloc((size_t)MsgLen);
                  if (pMsg == NULL)
                  {
                    MessageBox(NULL,
                               "Not enough storage",
                               "Storage error",
                               MB_OK);
                    return FALSE;
                  }
                  AllocatedStorage = 1;
                  CheckSum         = 0;
                                       /* Initialise with random chars*/
                  for (i=8; i<MsgLen ; i++)
                  {
                    pMsg[i]   = rand();
                    CheckSum += pMsg[i];
                  }
                                       /* Put header on               */
                  strcpy(pMsg,"[CK]");
                  * (long *) (pMsg+4) = CheckSum;
                }
                else
                {
                  AllocatedStorage = 0;
                  pMsg   = Msg;
                  MsgLen = strlen(Msg)+1;
                }
                                       /* Disable buttons             */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL),FALSE);
                EnableWindow(GetDlgItem(hDlg,IDOK)    ,FALSE);
                                       /* Issue the MQOPEN            */
                memcpy(ObjDesc.ObjectName,
                       pWords -> PutQName,
                 (int) MQ_Q_NAME_LENGTH);

                MQOPEN (pWords -> hConn,
                        &ObjDesc,
                        Options,
                        &hObj,
                        &CompCode,
                        &Reason);

                if (CompCode)
                {
                  MQIError(pWords,hDlg,"MQOPEN", CompCode, Reason);
                }
                else
                {
                  reps = atoi( CharReps );

                  if (pWords->PutSync) PutMsgOpts.Options |= MQPMO_SYNCPOINT;

                  memcpy(MsgDesc.Format,MQFMT_STRING,sizeof(MsgDesc.Format));

                  while (reps-- && CompCode == 0L)
                  {
                    MQPUT (pWords -> hConn,
                           hObj,
                          &MsgDesc,
                          &PutMsgOpts,
                           MsgLen,
                           pMsg,
                          &CompCode,
                          &Reason);

                    if (CompCode)
                    {
                      MQIError(pWords,hDlg,"MQPUT",CompCode, Reason);
                      break;
                    }
                    else
                    {
                                       /* Update count field          */
                      Count++;
                      _itoa(Count, Buffer, 10);
                      SetDlgItemText(hDlg, IDD_COUNT, Buffer);
                    }
                  }

                  MQCLOSE(pWords -> hConn,
                         &hObj,
                          0L,
                         &CompCode,
                         &Reason);
                  if (CompCode) MQIError(pWords,hDlg,"MQCLOSE", CompCode, Reason);
                }
                SetTitle("Put",hDlg,pWords);
                                       /* Free any allocated storage  */
                if (AllocatedStorage) free(pMsg);

                                       /* Enable buttons              */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL),TRUE);
                EnableWindow(GetDlgItem(hDlg,IDOK)    ,TRUE);
                return (TRUE);

           case IDCANCEL:
                EndDialog(hDlg, FALSE);
                return (TRUE);
         }
         break;
  }
  return (FALSE);
}

/**********************************************************************/
/*  FUNCTION: MQGetDlg                                                */
/*  PURPOSE:  Get dialog                                              */
/**********************************************************************/
BOOL FAR PASCAL MQGETDlg(HWND hDlg,
                         UINT message,
                         UINT wParam,
                         LONG lParam)
{
  HWND    hOwner;
  WORDS * pWords;
  MQLONG  CompCode, Reason;
  MQOD    ObjDesc    = { MQOD_DEFAULT };
  MQMD    MsgDesc    = { MQMD_DEFAULT };
  MQGMO   GetMsgOpts = { MQGMO_DEFAULT };
  MQLONG  Options    = MQOO_INPUT_SHARED;
  char    Msg[100];
  MQLONG  MsgLen,MsgSize;
  MQHOBJ  hObj;
  int     Count;
  char    Buffer[50];
  char  * pMsg       = NULL;
  int     AllocatedStorage;
  long    CheckSum;
  long    i;

  switch (message)
  {
    case WM_INITDIALOG:
                                       /* Find the Window words       */
         hOwner = GetWindow( hDlg, GW_OWNER );
         pWords = (WORDS *) GetWindowWord( hOwner, 0 );
         SetTitle("Get",hDlg,pWords);
                                       /* Set max size of field       */
         SendDlgItemMessage(hDlg,
                            IDD_Q,
                            EM_LIMITTEXT,
                      (int) MQ_Q_NAME_LENGTH,
                            0L);

                                       /* Set the field in the Dlg    */
         SetDlgItemText(hDlg,
                        IDD_Q,
                        pWords -> GetQName);

         SendDlgItemMessage(hDlg,
                            IDD_SYNCPOINT,
                            BM_SETCHECK, pWords -> GetSync,0);
         return (TRUE);
         break;

    case WM_USER_HELP:
         WinHelp(hDlg,HelpFileName,HELP_CONTEXT,IDM_MQGET);
         break;

    case WM_COMMAND:
         switch (wParam)
         {
           case IDD_NOWAIT:
           case IDD_WAIT:
                Count = 0;
                                       /* Find the Window words       */
                hOwner = GetWindow( hDlg, GW_OWNER );
                pWords = (WORDS *) GetWindowWord( hOwner, 0 );
                SetTitle("Getting...",hDlg,pWords);
                                       /* Disable buttons             */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL)  ,FALSE);
                EnableWindow(GetDlgItem(hDlg,IDD_NOWAIT),FALSE);
                EnableWindow(GetDlgItem(hDlg,IDD_WAIT)  ,FALSE);
                                       /* Get the field from the Dlg  */
                GetDlgItemText(hDlg,
                               IDD_Q,
                               pWords -> GetQName,
                               sizeof(pWords -> GetQName));

                pWords->GetSync = (int)SendDlgItemMessage(hDlg,
                                                          IDD_SYNCPOINT,
                                                          BM_GETCHECK, 0,0);
                                       /* Issue the MQOPEN            */
                memcpy(ObjDesc.ObjectName,
                       pWords -> GetQName,
                 (int) MQ_Q_NAME_LENGTH);

                MQOPEN (pWords -> hConn,
                        &ObjDesc,
                        Options,
                        &hObj,
                        &CompCode,
                        &Reason);

                if (CompCode)
                {
                  MQIError(pWords,hDlg,"MQOPEN", CompCode, Reason);
                }
                else
                {
                                       /* Did the user request a wait?*/
                  if (wParam == IDD_NOWAIT)
                  {
                    GetMsgOpts.Options      = MQGMO_NO_WAIT;
                  }
                  else
                  {
                    GetMsgOpts.Options      = MQGMO_WAIT;
                    GetMsgOpts.WaitInterval = MQWI_UNLIMITED;
                  }


                  if (pWords->GetSync) GetMsgOpts.Options |= MQGMO_SYNCPOINT;
                                  else GetMsgOpts.Options |= MQGMO_NO_SYNCPOINT;
                  GetMsgOpts.Options |= MQGMO_CONVERT;
                                                 /* Do get in local   */
                  pMsg             = Msg;
                  MsgSize          = sizeof(Msg)-1;
                  AllocatedStorage = 0;

                  while (CompCode != MQCC_FAILED && strcmp(Msg,"quit") )
                  {
                    memcpy(MsgDesc.MsgId   , MQMI_NONE, sizeof(MsgDesc.MsgId));
                    memcpy(MsgDesc.CorrelId, MQCI_NONE, sizeof(MsgDesc.CorrelId));

                    MQGET (pWords -> hConn,
                           hObj,
                           &MsgDesc,
                           &GetMsgOpts,
                            MsgSize,
                            pMsg,
                           &MsgLen,
                           &CompCode,
                           &Reason);
                    if (CompCode != MQCC_FAILED &&
                        Reason   != MQRC_TRUNCATED_MSG_FAILED)
                    {
                                       /* Is this a checksum message ?*/
                      if (memcmp(pMsg,"[CK]",4))
                      {
                                       /* No, can we display it ?     */
                        if (MsgLen < 100)
                                       /* Ensure NULL-terminated      */
                          pMsg[MsgLen]='\0';
                        else
                          wsprintf(pMsg,"%ld bytes received",MsgLen);
                      }
                      else
                      {
                                       /* Check the check sum         */
                        CheckSum = 0;
                        for (i=8 ; i<MsgLen ; i++)
                        {
                          CheckSum += pMsg[i];
                        }
                        if (CheckSum == *(long *) (pMsg+4))
                          wsprintf(pMsg,
                                  "%ld bytes received - Check sum matched",
                                   MsgLen);
                        else
                          wsprintf(pMsg,
                                  "%ld bytes received - Check sum failed",
                                   MsgLen);
                      }
                                       /* Add to Listbox (at the end) */
                      SendDlgItemMessage(hDlg,
                                         IDD_LIST,
                                         LB_INSERTSTRING, (UINT)-1,
                          (LONG) (LPSTR) pMsg);
                                       /* Update count field          */
                      Count++;
                      _itoa(Count, Buffer, 10);
                      SetDlgItemText(hDlg, IDD_COUNT, Buffer);
                    }
                    else
                    {
                      switch(Reason)
                      {
                        case MQRC_TRUNCATED_MSG_FAILED:
                                       /* Make our message area bigger*/
                             if (AllocatedStorage)
                             {
                               free(pMsg);
                               AllocatedStorage = 0;
                             }

                             pMsg   = malloc((size_t)MsgLen);
                             if (pMsg == NULL)
                             {
                               MessageBox(NULL,
                                          "Not enough storage",
                                          "Storage error",
                                          MB_OK);
                             }
                             else
                             {
                               MsgSize          = MsgLen;
                               AllocatedStorage = 1;
                               CompCode         = 0L;
                             }

                             break;

                        case MQRC_NO_MSG_AVAILABLE:
                             break;

                        default:
                             MQIError(pWords,hDlg,"MQPUT", CompCode, Reason);
                             break;
                      }
                    }
                  }

                  MQCLOSE(pWords -> hConn,
                         &hObj,
                          0L,
                         &CompCode,
                         &Reason);
                  if (CompCode) MQIError(pWords,hDlg,"MQCLOSE", CompCode, Reason);
                }
                SetTitle("Get",hDlg,pWords);
                                       /* Free allocated storage      */
                if (AllocatedStorage) free(pMsg);
                                       /* Enable buttons              */
                EnableWindow(GetDlgItem(hDlg,IDCANCEL)  ,TRUE);
                EnableWindow(GetDlgItem(hDlg,IDD_NOWAIT),TRUE);
                EnableWindow(GetDlgItem(hDlg,IDD_WAIT)  ,TRUE);

                return (TRUE);

           case IDCANCEL:
                EndDialog(hDlg, FALSE);
                return (TRUE);

           case IDD_CLEAR:
                                       /* Clear listbox               */
                SendDlgItemMessage(hDlg,
                                   IDD_LIST,
                                   LB_RESETCONTENT, 0,0);
                break;
         }
         break;
  }
  return (FALSE);
}

/**********************************************************************/
/*  FUNCTION: AboutDlg                                                */
/*  PURPOSE:  Display About Dialog                                    */
/**********************************************************************/
BOOL FAR PASCAL AboutDlg(HWND hDlg,
                         UINT message,
                         UINT wParam,
                         LONG lParam)
{
  switch (message)
  {
    case WM_INITDIALOG:
         return (TRUE);

    case WM_COMMAND:
         if (wParam == IDOK)
         {
           EndDialog(hDlg, TRUE);
           return (TRUE);
         }
         break;
  }
  return (FALSE);
}

/**********************************************************************/
/*  FUNCTION: MQIErrorDlg                                             */
/*  PURPOSE:  Display MQI Error                                       */
/**********************************************************************/
BOOL FAR PASCAL MQIErrorDlg(HWND hDlg,
                            UINT message,
                            UINT wParam,
                            LONG lParam)
{
  char Buffer[50];

  switch (message)
  {
    case WM_INITDIALOG:
                                       /* Set the fields in the Dlg   */
        SetDlgItemText(hDlg,
                       IDD_API,
                       GlobalApi);

        _ltoa(GlobalCompCode, Buffer, 10);
        SetDlgItemText(hDlg,
                       IDD_COMP,
                       Buffer);

        _ltoa(GlobalReason, Buffer, 10);
        SetDlgItemText(hDlg,
                       IDD_REASON,
                       Buffer);

         MessageBeep(MB_ICONEXCLAMATION);
         return (TRUE);

    case WM_USER_HELP:
         WinHelp(hDlg,HelpFileName,HELP_CONTEXT, IDH_MQERRORS);
         break;

    case WM_COMMAND:
         if (wParam == IDOK)
         {
           EndDialog(hDlg, TRUE);
           return (TRUE);
         }
         break;
  }
  return (FALSE);
}

/**********************************************************************/
/*  FUNCTION: MQIError                                                */
/*  PURPOSE:  Display error resulting from an MQ API call             */
/**********************************************************************/
static void MQIError(WORDS * pWords,
                     HWND    hWnd,
                     LPSTR   Api,
                     MQLONG  CompCode,
                     MQLONG  Reason)
{
  FARPROC     lpProc;
                                       /* Save data in statics        */
  GlobalApi      = Api;
  GlobalCompCode = CompCode;
  GlobalReason   = Reason;

  lpProc = MakeProcInstance(MQIErrorDlg, pWords -> hInstance);

  DialogBox(pWords -> hInstance,
            "MQERROR",
            hWnd,
            lpProc);

  FreeProcInstance(lpProc);
}

/**********************************************************************/
/*  FUNCTION: HelpMessageFilterHook                                   */
/*  PURPOSE:  Catch users issuing help                                */
/**********************************************************************/
DWORD FAR PASCAL HelpMessageFilterHook(int   nCode,
                                       WORD  wParam,
                                       LPMSG lpMsg)
{
    if (nCode < 0)  goto DefHook;
    if (!lpMsg)     goto DefHook;

    /******************************************************************/
    /* Was help during a dialog box                                   */
    /******************************************************************/
    if (nCode==MSGF_DIALOGBOX      &&
        lpMsg->message==WM_KEYDOWN &&
        lpMsg->wParam==VK_F1)
    {
      HWND hTemp=NULL;
      HWND hParent=lpMsg->hwnd;

      while (hParent != NULL)
      {
        hTemp=hParent;
        if (!(GetWindowLong(hTemp, GWL_STYLE) & WS_CHILD))
            break;
        hParent=GetWindowWord(hParent, GWW_HWNDPARENT);
      }

      if (hTemp) PostMessage(hTemp, WM_USER_HELP, nCode, 0L);
      return TRUE;
    }

    /******************************************************************/
    /* Was help during a menu                                         */
    /******************************************************************/
    if (nCode==MSGF_MENU           &&
        lpMsg->message==WM_KEYDOWN &&
        lpMsg->wParam==VK_F1)
    {
      PostMessage(hMain, WM_USER_HELP, nCode,
                  MAKELONG((WORD)lpMsg->hwnd,0));
      return TRUE;
    }

DefHook:
    return DefHookProc(nCode, wParam, (LONG)lpMsg, &NextMessageFilterProc);
}

/**********************************************************************/
/*  FUNCTION: SetTitle                                                */
/*  PURPOSE:  Set the title of a dialog                               */
/**********************************************************************/
static void     SetTitle   (LPSTR   Title,
                            HWND    hWnd,
                            WORDS * pWords)
{
  char Buffer[100];

  wsprintf(Buffer,"%s HCONN(%ld)",Title,pWords->hConn);
  SendMessage (hWnd,
               WM_SETTEXT,
               0,
 (LONG)(LPSTR) Buffer);
}

/**********************************************************************/
/*  FUNCTION: SetMenus                                                */
/*  PURPOSE:  Set the menus of the given window                       */
/**********************************************************************/
static void     SetMenus   (WORDS * pWords)
{
  HMENU hMenu;
  hMenu = GetMenu(pWords -> hWnd);
  if (pWords->hConn == MQHC_UNUSABLE_HCONN)
  {
    EnableMenuItem(hMenu, IDM_MQDISC , MF_DISABLED | MF_GRAYED);
    EnableMenuItem(hMenu, IDM_MQPUT  , MF_DISABLED | MF_GRAYED);
    EnableMenuItem(hMenu, IDM_MQGET  , MF_DISABLED | MF_GRAYED);
    EnableMenuItem(hMenu, IDM_MQCMIT , MF_DISABLED | MF_GRAYED);
    EnableMenuItem(hMenu, IDM_MQBACK , MF_DISABLED | MF_GRAYED);
  }
  else
  {
    EnableMenuItem(hMenu, IDM_MQDISC , MF_ENABLED);
    EnableMenuItem(hMenu, IDM_MQPUT  , MF_ENABLED);
    EnableMenuItem(hMenu, IDM_MQGET  , MF_ENABLED);
    EnableMenuItem(hMenu, IDM_MQCMIT , MF_ENABLED);
    EnableMenuItem(hMenu, IDM_MQBACK , MF_ENABLED);
  }
}

/**********************************************************************/
/*  FUNCTION: GetHelpName                                             */
/*  PURPOSE:  Get the fully qualified location of help file           */
/**********************************************************************/
void GetHelpName(HANDLE   hInstance,
                 char   * HelpFileName)
{
   int Len;
                                       /* Where is the EXE ?          */
   Len = GetModuleFileName(hInstance,HelpFileName,HELP_FILE_NAME_SIZE);
   HelpFileName += Len-1;

   while (*HelpFileName != '\\' && *HelpFileName != ':') HelpFileName--;
   strcpy(HelpFileName+1, "MQCL.HLP");
}
